use serde::{Deserialize, Serialize};
use std::collections::HashMap;
use std::error::Error;

#[derive(Debug, Deserialize)]
#[serde(rename_all = "PascalCase")]
struct PollenApiResponse {
    forecast_date: String,
    location: Option<Location>,
}

#[derive(Debug, Deserialize)]
struct Location {
    periods: Vec<Period>,
    #[serde(rename = "DisplayLocation")]
    display_location: String,
}

#[derive(Debug, Deserialize)]
#[serde(rename_all = "PascalCase")]
struct Period {
    r#type: String,
    index: f32,
    triggers: Vec<Trigger>,
}

#[derive(Debug, Deserialize)]
#[serde(rename_all = "PascalCase")]
struct Trigger {
    name: String,
    #[allow(dead_code)]
    plant_type: String,
    #[allow(dead_code)]
    genus: String,
}

#[derive(Serialize, Deserialize)]
pub struct PollenPeriod {
    pub index: f32,
    pub triggers: Vec<String>,
}

#[derive(Serialize, Deserialize)]
pub struct PollenSummary {
    pub location: String,
    pub forecast_date: String,
    pub periods: HashMap<String, PollenPeriod>, // "Yesterday", "Today", "Tomorrow"
}

pub async fn fetch_pollen_api(zip: &str) -> Result<PollenSummary, Box<dyn Error>> {
    let url = format!("https://www.pollen.com/api/forecast/current/pollen/{}", zip);
    let resp = reqwest::Client::new()
        .get(&url)
        .header("User-Agent", "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/123.0.0.0 Safari/537.36")
        .header("Accept", "application/json, text/plain, */*")
        .header("Referer", &url)
        .header("Cookie", format!("geo={}", zip))
        .send().await?
        .text().await?;

    let api: PollenApiResponse = serde_json::from_str(&resp)?;
    let loc = api.location.ok_or("No location in pollen.com response")?;
    let mut periods = HashMap::new();

    for period in &loc.periods {
        let triggers: Vec<String> = period.triggers.iter().map(|t| t.name.clone()).collect();
        periods.insert(
            period.r#type.clone(),
            PollenPeriod {
                index: period.index,
                triggers,
            },
        );
    }

    Ok(PollenSummary {
        location: loc.display_location,
        forecast_date: api.forecast_date,
        periods,
    })
}
